==> apache-arrow-go-18.5.0.tar.gz: apache-arrow-go-18.5.0/arrow/cdata/import_allocator.go <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cdata

import (
	"sync/atomic"
	"unsafe"

	"github.com/apache/arrow-go/v18/arrow/internal/debug"
)

// #include "helpers.h"
// #include <stdlib.h>
import "C"

type importAllocator struct {
	bufCount atomic.Int64

	arr *CArrowArray
}

func (i *importAllocator) addBuffer() {
	i.bufCount.Add(1)
}

func (*importAllocator) Allocate(int) []byte {
	panic("cannot allocate from importAllocator")
}

func (*importAllocator) Reallocate(int, []byte) []byte {
	panic("cannot reallocate from importAllocator")
}

func (i *importAllocator) Free([]byte) {
	debug.Assert(i.bufCount.Load() > 0, "too many releases")

	if i.bufCount.Add(-1) == 0 {
		defer C.free(unsafe.Pointer(i.arr))
		C.ArrowArrayRelease(i.arr)
		if C.ArrowArrayIsReleased(i.arr) != 1 {
			panic("did not release C mem")
		}
	}
}

==> apache-arrow-go-18.5.1.tar.gz: apache-arrow-go-18.5.1/arrow/cdata/import_allocator.go <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cdata

import (
	"sync/atomic"
	"unsafe"

	"github.com/apache/arrow-go/v18/arrow/internal/debug"
)

// #include "helpers.h"
// #include <stdlib.h>
import "C"

type importAllocator struct {
	bufCount atomic.Int64
	released atomic.Bool

	arr *CArrowArray
}

func (i *importAllocator) addBuffer() {
	i.bufCount.Add(1)
}

func (*importAllocator) Allocate(int) []byte {
	panic("cannot allocate from importAllocator")
}

func (*importAllocator) Reallocate(int, []byte) []byte {
	panic("cannot reallocate from importAllocator")
}

func (i *importAllocator) Free([]byte) {
	debug.Assert(i.bufCount.Load() > 0, "too many releases")

	if i.bufCount.Add(-1) == 0 {
		i.forceRelease()
	}
}

func (i *importAllocator) forceRelease() {
	if i.released.CompareAndSwap(false, true) {
		defer C.free(unsafe.Pointer(i.arr))
		C.ArrowArrayRelease(i.arr)
		if C.ArrowArrayIsReleased(i.arr) != 1 {
			panic("did not release C mem")
		}
	}
}
